-------------------------------------------------------------------------------
--|> Bars
-- Manages the display of the bars and their elements representing detected players
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP Globals
-- luacheck: globals weizPVP weizPVP.Settings
--> Addon Libs Globals
-- luacheck: globals LibStub ViragDevTool ViragDevTool_AddData
--> WoW Globals
-- luacheck: globals CreateFrame UIParent RAID_CLASS_COLORS InCombatLockdown GetUnitName
-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--|> GLOBALS
-------------------------------------------------------------------------------
weizPVP.Bars = {}
weizPVP.Buttons = {}
weizPVP.Icons = {}

--=> GENERATE BARS IN LIST FRAME
--> Creates a set of test bars to be placed on the ListFrame
--> These bars will aw=lways be generated when the addon starts so threre isn'ta need to create more whiile in combat, which is restricted by he WoW API
--> Also a good example for a testing mode to use later on.
-------------------------------------------------------------------------------
function weizPVP:CreateBars()
    local barNumber = 1 --> Start naming @ #1

    -------------------------------------------------------------------------------
    --=> Create all displayable bars
    --!! Creating all of the bars allows you to update the bars during combat, otherwise its a protected API function
    -------------------------------------------------------------------------------
    for k = 1, weizPVP.Settings.BARS.MAXNUMBER do -- MAKE THE MAX AMOUNT OF BUTONS from BARS.MAXNUMBER
        --=> CREATE BAR
        -----------------------------------------------------------------------
        local row = CreateFrame("StatusBar", "weizPVP.Bar" .. barNumber, weizPVP.ListFrame)
        row.NameText = row:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        row.LevelText = row:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        row.GuildText = row:CreateFontString(nil, "ARTWORK", "GameFontNormal")
        row.id = barNumber
        row.name = ""

        --=> BAR ICONS
        row.TargetIcon = row:CreateTexture(nil, "OVERLAY", nil)
        row.DEADIcon = row:CreateTexture(nil, "OVERLAY", nil)

        if barNumber == 1 then -->If  its's the first bar, just anchor it to the top
            row:SetPoint("TOPLEFT", weizPVP.ListFrame, "TOPLEFT")
        else --> Bars after 1 using an offset t spacee themsleves out
            row:SetPoint(
                "TOPLEFT",
                weizPVP.ListFrame,
                "TOPLEFT",
                0,
                -1 * (weizPVP.Settings.LIST.ROWHEIGHT * (barNumber - 1))
            )
        end

        --> Set properties
        row:SetHeight(weizPVP.Settings.BARS.HEIGHT)
        row:SetPoint("RIGHT", weizPVP.ScrollFrame, "RIGHT")
        row:SetPoint("LEFT", weizPVP.ScrollFrame, "LEFT")
        row:SetStatusBarTexture(SM:Fetch("statusbar", weizPVP.Settings.BARS.TEXTURE))
        row:SetStatusBarColor(0, 0, 0, 0)
        row:SetMinMaxValues(0, 100)
        row:EnableMouse(true)
        row:SetValue(100)

        --> Set Background
        row.bg = row:CreateTexture(nil, "BACKGROUND")
        row.bg:SetTexture(SM:Fetch("statusbar", weizPVP.Settings.BARS.BGTEXTURE))
        row.bg:SetAllPoints(true)
        row.bg:SetVertexColor(0, 0, 0, 0)

        --=> Name Text
        -----------------------------------------------------------------------
        row.NameText:SetFont(
            SM:Fetch("font", weizPVP.Settings.FONTS.NARROW),
            weizPVP.Settings.BARS.HEIGHT + weizPVP.Settings.BARS.NAME_FONT_DIFF,
            "OUTLINE"
        )
        row.NameText:SetPoint("TOPLEFT", row, "TOPLEFT", 2, 0)
        row.NameText:SetPoint("BOTTOMLEFT", row, "BOTTOMLEFT", 2, 1)
        row.NameText:SetHeight(weizPVP.Settings.BARS.HEIGHT)
        row.NameText:SetJustifyH("LEFT")
        row.NameText:SetJustifyV("MIDDLE")
        row.NameText:SetTextColor(1, 1, 1, 1)
        row.NameText:SetShadowColor(0, 0, 0, 0.8)

        --=> Level Text
        -----------------------------------------------------------------------
        row.LevelText:SetFont(
            SM:Fetch("font", weizPVP.Settings.FONTS.ITALIC),
            weizPVP.Settings.BARS.HEIGHT + weizPVP.Settings.BARS.LEVEL_FONT_DIFF,
            "OUTLINE"
        )
        row.LevelText:SetPoint("TOPLEFT", row.NameText, "TOPRIGHT", 0, 0)
        row.LevelText:SetPoint("BOTTOMLEFT", row.NameText, "BOTTOMRIGHT", 0, -1)
        row.LevelText:SetJustifyH("LEFT")
        row.LevelText:SetJustifyV("MIDDLE")
        row.LevelText:SetTextColor(1, 0.8, 0.8, 1)
        row.LevelText:SetShadowColor(0, 0, 0, 0.8)

        --=> Guild Text
        -----------------------------------------------------------------------
        row.GuildText:SetFont(
            SM:Fetch("font", weizPVP.Settings.FONTS.NARROW),
            weizPVP.Settings.BARS.HEIGHT + weizPVP.Settings.BARS.GUILD_FONT_DIFF,
            "OUTLINE"
        )
        row.GuildText:SetPoint("TOPRIGHT", row, "TOPRIGHT", -2, 0)
        row.GuildText:SetPoint("BOTTOMRIGHT", row, "BOTTOMRIGHT", -2, -1)
        row.GuildText:SetJustifyH("RIGHT")
        row.GuildText:SetJustifyV("CENTER")
        row.GuildText:SetTextColor(1, 1, 1, 1)
        row.GuildText:SetShadowColor(0, 0, 0, 0.6)

        --=> KOS Icon
        -----------------------------------------------------------------------
        -- row.TargetIcon:SetPoint("LEFT", row, "LEFT", 0, 0)
        row.TargetIcon:SetPoint("RIGHT", row.NameText, "LEFT", 16, 0)
        row.TargetIcon:SetSize(weizPVP.Settings.BARS.HEIGHT, weizPVP.Settings.BARS.HEIGHT)
        row.TargetIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\indicator-target.tga", false)
        row.TargetIcon:Hide()

        --=> DEAD Icon
        -----------------------------------------------------------------------
        row.DEADIcon:SetPoint("LEFT", row.LevelText, "RIGHT", 4, 0)
        row.DEADIcon:SetSize(weizPVP.Settings.BARS.HEIGHT, weizPVP.Settings.BARS.HEIGHT)
        row.DEADIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\icon_dead.tga", false)
        row.DEADIcon:Hide()

        --=> SECURE BUTTON for registering clicks
        -------------------------------------------------------------------------------
        local secureButton = CreateFrame("Button", "weizPVP.SecureButton" .. barNumber, row, "weizPVPSecureButton")
        secureButton:SetPoint("TOPLEFT", row, "TOPLEFT")
        secureButton:SetPoint("BOTTOMRIGHT", row, "BOTTOMRIGHT")
        secureButton:SetAlpha(0)
        secureButton:Show()
        secureButton.id = barNumber

        --=> Set Globals
        -----------------------------------------------------------------------
        weizPVP.Bars[barNumber] = row
        weizPVP.Buttons[barNumber] = secureButton

        --=> SbarNumber++
        -----------------------------------------------------------------------
        barNumber = barNumber + 1
    end
end

-------------------------------------------------------------------------------
--=> Row Clicked: Secure Button
-------------------------------------------------------------------------------
--> PRECLICK
function weizPVP:ButtonPreClick(self, button)
    local barName = _G["weizPVP.Bar" .. tostring(self.id)]
    local playerName = barName.name
    if playerName and playerName ~= "" then
        if button == "LeftButton" then
            if not InCombatLockdown() then
                self:SetAttribute("macrotext", "/target " .. playerName)
            -- else

            --     if self:GetAttribute("macrotext") ~= "/target " .. playerName then --> INCOMBAT; WRONG TARGET
            --         -- self:EnableMouse(false)
            --     end
            end
        end
    end
end

--> POSTCLICK
-- local lastTargetBarID
function weizPVP:ButtonPostClick(self, button)
    -- local barName = _G["weizPVP.Bar" .. tostring(self.id)]
    local bar = weizPVP.Bars[self.id]
    local playerName = bar.name

    if playerName and playerName ~= "" then
        if button == "LeftButton" then
            if not InCombatLockdown() then
                local targetName = GetUnitName("target", true)
                if
                    targetName ~= playerName and weizPVP.ActiveList[playerName] and
                        weizPVP.PlayerActiveCache[playerName].Stealth ~= true
                 then
                    weizPVP.InactiveList[playerName] = weizPVP.ActiveList[playerName]
                    -- weizPVP.InactiveList[playerName].TimeAdded =
                    --     weizPVP.ActiveList[playerName].TimeAdded + weizPVP.PLAYER_ACTIVE_TIMEOUT
                        -- weizPVP.CurrentList[playerName].TimeAdded = weizPVP.ActiveList[playerName].TimeAdded + weizPVP.PLAYER_ACTIVE_TIMEOUT
                        -- weizPVP.NearbyList[playerName].TimeAdded = weizPVP.ActiveList[playerName].TimeAdded + weizPVP.PLAYER_ACTIVE_TIMEOUT
                    weizPVP.ActiveList[playerName] = nil
                    weizPVP:SortNearbyList()
                    weizPVP:RefreshCurrentList()
                    -- weizPVP:ManageExpirations()
                    -- weizPVP:UpdateNearbyCount()
                -- else
                -- bar.TargetIcon:Show()
                -- bar.NameText:SetPoint("LEFT", 16, 0);
                -- if lastTargetBarID then
                --     local lastTargetBar = weizPVP.Bars[lastTargetBarID]
                --     lastTargetBar.TargetIcon:Hide()
                -- end
                -- lastTargetBarID = self.id
                end
            end
        end
    end
end
