-------------------------------------------------------------------------------
--|> Buttons
-- Manages the butons on the frame that will perform certain functions
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP Globals
-- luacheck: globals weizPVP weizPVP.Settings
--> Addon Libs Globals
-- luacheck: globals LibStub
--> WoW Globals
-- luacheck: globals CreateFrame UIFrameFadeOut UIFrameFadeIn InCombatLockdown
-------------------------------------------------------------------------------


--=> Drag Bottom Right
-------------------------------------------------------------------------------
function weizPVP:CreateRezieCorner()
    local DragBottomRight = CreateFrame("Button", "weizPVPResizeGripRight", weizPVP.CoreFrame)
    DragBottomRight:SetFrameLevel(6)
    DragBottomRight:SetNormalTexture("Interface\\AddOns\\weizPVP\\Icons\\resizeGrip.tga")
    DragBottomRight:SetHighlightTexture("Interface\\AddOns\\weizPVP\\Icons\\resizeGripHighlight.tga")
    DragBottomRight:SetWidth(16)
    DragBottomRight:SetHeight(16)
    DragBottomRight:SetPoint("BOTTOMRIGHT", weizPVP.FooterFrame, "BOTTOMRIGHT")
    DragBottomRight:Show()

    --> DRAG RESIZE: OnMouseDown
    -----------------------------------
    DragBottomRight:SetScript(
        "OnMouseDown",
        function(self, button)
            if ((not weizPVP.CoreFrame.isLocked) or weizPVP.CoreFrame.isLocked == 0) and (button == "LeftButton") then
                weizPVP.CoreFrame.isResizing = true
                weizPVP.CoreFrame:StartSizing("BOTTOMRIGHT")
            end
        end
    )

    --> DRAG RESIZE: OnMouseUp
    -----------------------------------
    DragBottomRight:SetScript(
        "OnMouseUp",
        function(self, button)
            if weizPVP.CoreFrame.isResizing == true and (button == "LeftButton") then
                weizPVP.CoreFrame:StopMovingOrSizing()
                weizPVP.CoreFrame.isResizing = false
            end
            weizPVP:SortNearbyList()
            weizPVP:RefreshCurrentList()
        end
    )

    --> DRAG RESIZE: OnMouseUp
    -----------------------------------
    weizPVP.HeaderFrame.ClearButton:SetScript(
        "OnMouseDown",
        function(self, event)
            if event == "LeftButton" then
                weizPVP:ClearListData()
            end
        end
    )
end

function weizPVP:CreateHeaderIcons()
    --=> weizPVP.HeaderFrame.MinButton: ICON
    -------------------------------------------------------------------------------
    weizPVP.HeaderFrame.MinButton = CreateFrame("Button", "weizPVP.HeaderFrame.MinButton", weizPVP.HeaderFrame)
    weizPVP.HeaderFrame.MinButton:SetPoint("TOPRIGHT", weizPVP.HeaderFrame, "TOPRIGHT", -8, -1)
    weizPVP.HeaderFrame.MinButton:SetSize(16, 16)
    weizPVP.HeaderFrame.MinButton:SetNormalTexture("Interface\\addons\\weizPVP\\Icons\\arrow-up.tga")
    weizPVP.HeaderFrame.MinButton:GetNormalTexture():SetVertexColor(0.3, 0.3, 0.3)
    weizPVP.HeaderFrame.MinButton:SetHighlightTexture("Interface\\addons\\weizPVP\\Icons\\arrow-up-highlight.tga")

    --=> Expand Button; restores frame to size before using MinButton
    -------------------------------------------------------------------------------
    weizPVP.HeaderFrame.MaxButton = CreateFrame("Button", "weizPVP.HeaderFrame.MaxButton", weizPVP.HeaderFrame)
    weizPVP.HeaderFrame.MaxButton:SetPoint("BOTTOMRIGHT", weizPVP.HeaderFrame, "BOTTOMRIGHT", -8, 1)
    weizPVP.HeaderFrame.MaxButton:SetSize(16, 16)
    weizPVP.HeaderFrame.MaxButton:SetNormalTexture("Interface\\addons\\weizPVP\\Icons\\arrow-down.tga")
    weizPVP.HeaderFrame.MaxButton:SetHighlightTexture("Interface\\addons\\weizPVP\\Icons\\arrow-down-highlight.tga")
    weizPVP.HeaderFrame.MaxButton:GetNormalTexture():SetVertexColor(0.2, 0.8, 1)

    --> MIN/MAX BUTTON: OnMouseDown
    -----------------------------------
    local minimized = false

    weizPVP.HeaderFrame.MinButton:SetScript(
        "OnMouseDown",
        function(self, event)
            if not InCombatLockdown() then --> added until we fix frames
                if event == "LeftButton" then
                    if minimized == false then
                        weizPVP.HeaderFrame.MinButton:GetNormalTexture():SetVertexColor(0.2, 0.8, 1)
                        weizPVP.HeaderFrame.MaxButton:GetNormalTexture():SetVertexColor(0.3, 0.3, 0.3)
                        UIFrameFadeOut(weizPVP.CoreFrame, 0.1, 1, 0)
                        weizPVP:ScheduleTimer("CoreWindowHide", 0.1)
                        minimized = true
                    end
                end
            end
        end
    )

    weizPVP.HeaderFrame.MaxButton:SetScript(
        "OnMouseDown",
        function(self, event)
            if not InCombatLockdown() then --> added until we fix frames
                if event == "LeftButton" then
                    if minimized == true then
                        weizPVP.HeaderFrame.MinButton:GetNormalTexture():SetVertexColor(0.3, 0.3, 0.3)
                        weizPVP.HeaderFrame.MaxButton:GetNormalTexture():SetVertexColor(0.2, 0.8, 1)
                        UIFrameFadeIn(weizPVP.CoreFrame, 0.1, 0, 1)
                        weizPVP.CoreFrame:Show()
                        minimized = false
                    end
                end
            end
        end
    )

    --=> Clear List Button
    -------------------------------------------------------------------------------
    weizPVP.HeaderFrame.ClearButton = CreateFrame("Button", "weizPVPClearButton", weizPVP.HeaderFrame)
    weizPVP.HeaderFrame.ClearButton:SetPoint("TOPRIGHT", weizPVP.HeaderFrame.MinButton, "TOPLEFT", -2, 0)
    weizPVP.HeaderFrame.ClearButton:SetSize(32, 32)
    weizPVP.HeaderFrame.ClearButton:SetNormalTexture("Interface\\addons\\weizPVP\\Icons\\button_refresh.tga")
    weizPVP.HeaderFrame.ClearButton:SetHighlightTexture(
        "Interface\\addons\\weizPVP\\Icons\\button_refresh_highlight.tga"
    )
end

function weizPVP:CoreWindowHide()
    weizPVP.CoreFrame:Hide()
end
