-------------------------------------------------------------------------------
--|> StealthAlert
-- A popup frame dedicated to stealth detection
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP Globals
-- luacheck: globals weizPVP
--> Addon Libs Globals
-- luacheck: globals LibStub ViragDevTool_AddData
--> WoW Globals
-- luacheck: globals CreateFrame UIParent PlaySoundFile C_Timer
------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
--> Get
local topOffset = UIParent:GetTop() / 3

--|> BUILD FRAME
-----------------------------------------------------------
weizPVP.StealthAlertFrame = CreateFrame("Frame", "weizPVP.StealthAlertFrame", UIParent)
weizPVP.StealthAlertFrame:SetFrameStrata("HIGH")
weizPVP.StealthAlertFrame:SetPoint("TOP", UIParent, "CENTER", 0, topOffset + 60)
weizPVP.StealthAlertFrame:SetHeight(90)
weizPVP.StealthAlertFrame:SetWidth(160)
weizPVP.StealthAlertFrame:SetBackdrop(
    {
        bgFile = SM:Fetch("background", "weizPVP-Background"),
        tile = false,
        edgeFile = SM:Fetch("border", "weizPVP-Border"),
        edgeSize = 2,
        insets = {left = 0, right = 0, top = 0, bottom = 0}
    }
)
weizPVP.StealthAlertFrame:SetBackdropColor(0, 0, 0, 0.6)
weizPVP.StealthAlertFrame:SetBackdropBorderColor(0, 0, 0, 0.8)
weizPVP.StealthAlertFrame:SetFrameLevel(2)
weizPVP.StealthAlertFrame:Hide()

--=> ModelBG
-----------------------------------------------------------
weizPVP.Settings.StealthAlertFrameModelPath = "spells/saberon_incense_state_purple.m2"
weizPVP.AlertModelBG = CreateFrame("CinematicModel", "weizPVP_AlertModelBG", weizPVP.StealthAlertFrame)
weizPVP.AlertModelBG:SetPoint("CENTER", weizPVP.StealthAlertFrame, "CENTER")
weizPVP.AlertModelBG:SetWidth(weizPVP.StealthAlertFrame:GetWidth() - 4)
weizPVP.AlertModelBG:SetHeight(weizPVP.StealthAlertFrame:GetHeight() - 4)
weizPVP.AlertModelBG:SetCamera(0)
weizPVP.AlertModelBG:SetModel(weizPVP.Settings.StealthAlertFrameModelPath)
weizPVP.AlertModelBG:SetAlpha(0.8)
weizPVP.AlertModelBG:SetFrameLevel(1)

--=> StealthAlertFrame.TextTop
-----------------------------------------------------------
weizPVP.StealthAlertFrame.TextTop = weizPVP.StealthAlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
weizPVP.StealthAlertFrame.TextTop:SetPoint("TOP", weizPVP.StealthAlertFrame, "TOP", 0, -8)
weizPVP.StealthAlertFrame.TextTop:SetFont(SM:Fetch("font", "Accidental Presidency"), 26, "OUTLINE")
weizPVP.StealthAlertFrame.TextTop:SetJustifyH("CENTER")
weizPVP.StealthAlertFrame.TextTop:SetTextColor(1, 0.3, 0.9, 1)
weizPVP.StealthAlertFrame.TextTop:SetText("STEALTH DETECTED")
weizPVP.StealthAlertFrame.TextTop:Hide()

--=> StealthAlertFrame.TextBottom
-----------------------------------------------------------
weizPVP.StealthAlertFrame.TextBottom = weizPVP.StealthAlertFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
weizPVP.StealthAlertFrame.TextBottom:SetPoint("TOP", weizPVP.StealthAlertFrame.TextTop, "BOTTOM", 0, -4)
weizPVP.StealthAlertFrame.TextBottom:SetFont(SM:Fetch("font", "Roboto Condensed Bold"), 24, "OUTLINE")
weizPVP.StealthAlertFrame.TextBottom:SetJustifyH("CENTER")
weizPVP.StealthAlertFrame.TextBottom:SetTextColor(1, 1, 1, 1)
weizPVP.StealthAlertFrame.TextBottom:SetText("PLAYERNAME")
weizPVP.StealthAlertFrame.TextBottom:Hide()

--=> Stealth Alert
-----------------------------------------------------------
local playSound = true
local function EnableSound()
    playSound = true
end
--> TEST VIA CHAT: /run weizPVP:ShowStealthAlert("STEALTH DETECTED", "Playername")
function weizPVP:ShowStealthAlert(textTop, textBottom)
    if weizPVP.StealthAlertFrame.anim then
        weizPVP.StealthAlertFrame.anim:Stop()
    else
        weizPVP.StealthAlertFrame.anim = weizPVP.StealthAlertFrame:CreateAnimationGroup()
        weizPVP.StealthAlertFrame.anim:SetScript(
            "OnFinished",
            function()
                weizPVP.StealthAlertFrame:Hide()
            end
        )

        local fade1 = weizPVP.StealthAlertFrame.anim:CreateAnimation("Alpha")
        fade1:SetDuration(0.2)
        fade1:SetFromAlpha(0)
        fade1:SetToAlpha(1)
        fade1:SetEndDelay(3) --> Set the number of seconds the animation delays after finishing.
        fade1:SetOrder(1)

        local fade2 = weizPVP.StealthAlertFrame.anim:CreateAnimation("Alpha")
        fade2:SetDuration(0.4)
        fade2:SetFromAlpha(1)
        fade2:SetToAlpha(0)
        fade2:SetOrder(2)
    end

    weizPVP.StealthAlertFrame.TextTop:SetText(textTop)
    weizPVP.StealthAlertFrame.TextBottom:SetText(textBottom)

    --> Set Background to resize based on text boundries
    local frameWidth
    local frameHeight = weizPVP.StealthAlertFrame:GetTop() - weizPVP.StealthAlertFrame.TextBottom:GetBottom() + 8
    weizPVP.StealthAlertFrame:SetHeight(frameHeight)

    if weizPVP.StealthAlertFrame.TextTop:GetWidth() > weizPVP.StealthAlertFrame.TextBottom:GetWidth() then
        frameWidth = weizPVP.StealthAlertFrame.TextTop:GetWidth()
    else
        frameWidth = weizPVP.StealthAlertFrame.TextBottom:GetWidth()
    end

    weizPVP.StealthAlertFrame:SetWidth(frameWidth + 48)
    weizPVP.AlertModelBG:SetWidth(weizPVP.StealthAlertFrame:GetWidth() + 4)
    weizPVP.AlertModelBG:SetHeight(weizPVP.StealthAlertFrame:GetHeight() + 4)

    --> Set Model
    weizPVP.AlertModelBG:SetModel(weizPVP.Settings.StealthAlertFrameModelPath)
    --> Show Elements
    weizPVP.StealthAlertFrame.TextTop:Show()
    weizPVP.StealthAlertFrame.TextBottom:Show()
    weizPVP.StealthAlertFrame:SetAlpha(0)
    weizPVP.StealthAlertFrame:Show()
    weizPVP.StealthAlertFrame.anim:Play()
    --> play stealth alert sound
    if playSound then
        PlaySoundFile("Interface\\AddOns\\weizPVP\\Sounds\\alert-1.ogg")
        playSound = false
        C_Timer.After(3, EnableSound)
    end
end
