-------------------------------------------------------------------------------
--=> Optimizer.lua
-- Deals with things related to keeps the addon and game running smoothly while under stress
-------------------------------------------------------------------------------
--=> LUACHECK OPTIONS
-------------------------------------------------------------------------------
--> LuaCheck Settings-
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP Globals
-- luacheck: globals weizPVP
--> Addon Libs Globals
-- luacheck: globals
--> WoW Globals
-- luacheck: globals SetCVar GetFramerate GetNetStats GetCVar
-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------
weizPVP.ParticleDensity = GetCVar("particleDensity") -- 10 to 100
weizPVP.Latency = select(4, GetNetStats()) -- World Latency

weizPVP.ParseData = true
weizPVP.LowFPS = false

--=> High Usage Check
--> Checks framerate and number of active players to determine if we're lagging duh to large number fights
local fpsSamples = {
    ["a"] = 60,
    ["b"] = 60,
    ["c"] = 60,
    ["d"] = 60
}
local fpsAverage = 60
local latencySamples = {
    ["a"] = 60,
    ["b"] = 60,
    ["c"] = 60,
    ["d"] = 60
}
local latencyAverage = 60



-------------------------------------------------------------------------------
function weizPVP:HighUsageCheck()
    --> Determine if we should still be parsing new player data
    if
        weizPVP.Settings.ResourceUsage.MaxShownPlayers and
            weizPVP.Settings.ResourceUsage.MaxShownPlayers > weizPVP.NearbyCount
     then
        weizPVP.ParseData = true
    else
        weizPVP.ParseData = false
    end

    --> Set Usage High if low FPS and lots of detected players
    local framerate = GetFramerate()
    if framerate < 20 and weizPVP.ParseData then
        weizPVP.Settings.ResourceUsage.High = true
    else
        weizPVP.Settings.ResourceUsage.High = false
    end

    --> Update Average FPS
    fpsSamples.d = fpsSamples.c
    fpsSamples.c = fpsSamples.b
    fpsSamples.b = fpsSamples.a
    fpsSamples.a = GetFramerate()

    fpsAverage = ((fpsSamples.a + fpsSamples.b + fpsSamples.c + fpsSamples.d) / 4)
    -- print ("average fps is: " .. fpsAverage)

        --> Update Average Latency
        latencySamples.d = latencySamples.c
        latencySamples.c = latencySamples.b
        latencySamples.b = latencySamples.a
        latencySamples.a = select(4, GetNetStats())

        latencyAverage = ((latencySamples.a + latencySamples.b + latencySamples.c + latencySamples.d) / 4)
        -- print ("Average World ms is: " .. latencyAverage)


    weizPVP.Settings.FPS = fpsAverage
    weizPVP.Settings.Latency = latencyAverage
    --> Adjust CVars if it looks really bad

    -- if fpsAverage < 20 then
    --     SetCVar("particleDensity", 10)
    -- elseif GetCVar("particleDensity") ~= 10 then
    --     SetCVar("particleDensity", weizPVP.ParticleDensity)
    -- end
end
