-------------------------------------------------------------------------------
--|> Crosshair
-- Place a crosshair on target's nameplate if they're a hostile player
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> weizPVP Globals
-- luacheck: globals weizPVP weizPVP.Settings
--> Addon Libs Globals
-- luacheck: globals LibStub
--> WoW Globals
-- luacheck: globals CreateFrame UIParent RAID_CLASS_COLORS InCombatLockdown UnitSelectionColor GetTime WorldFrame IsFlying
-- luacheck: globals UnitClass UnitIsPlayer IsItemInRange C_NamePlate IsUsableItem UnitIsEnemy UnitIsUnit GetPhysicalScreenSize
-------------------------------------------------------------------------------
--|> DEBUG
-------------------------------------------------------------------------------
--!! true = Targets all players
--!! false = Targets only hostile players (DEFAULT)
local DEBUG = false

-------------------------------------------------------------------------------
--|> GLOBALS
-------------------------------------------------------------------------------
weizPVP.Options["Crosshair"] = {}
weizPVP.Options["Crosshair"].Enabled = true

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
local targetPlate
local lineAlpha = 0.2
local circleAlpaha = 0.5
local arrowAlpha = 1

-------------------------------------------------------------------------------
--|> CREATE UI ELEMENTS
-------------------------------------------------------------------------------

--=> CREATE CROSSHAIR FRAME
-------------------------------------------------------
weizPVP.CrosshairFrame = CreateFrame("frame", "weizPVP:CrosshairFrame", WorldFrame)
weizPVP.CrosshairFrame:Hide()
weizPVP.CrosshairFrame:SetFrameLevel(0)
weizPVP.CrosshairFrame:SetFrameStrata("BACKGROUND")
weizPVP.CrosshairFrame:SetPoint("CENTER")
weizPVP.CrosshairFrame:SetSize(64, 64)

--=> CREATE CIRCLE TEXTURE
-------------------------------------------------------
local circle = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
circle:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/circle.tga]])
circle:SetAllPoints()
circle:SetBlendMode("ADD")
circle:SetAlpha(circleAlpaha)

--=> CREATE LINES
-------------------------------------------------------
--> LEFT
local leftLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
leftLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
leftLine:SetThickness(2)
leftLine:SetStartPoint("LEFT", weizPVP.CrosshairFrame, -2000, 0)
leftLine:SetEndPoint("LEFT", weizPVP.CrosshairFrame, 8, 0)
leftLine:SetVertexColor(1, 1, 1)
leftLine:SetAlpha(lineAlpha)
leftLine:SetBlendMode("ADD")
--> RIGHT
local rightLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
rightLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
rightLine:SetThickness(2)
rightLine:SetStartPoint("RIGHT", weizPVP.CrosshairFrame, 2000, 0)
rightLine:SetEndPoint("RIGHT", weizPVP.CrosshairFrame, -8, 0)
rightLine:SetVertexColor(1, 1, 1)
rightLine:SetAlpha(lineAlpha)
rightLine:SetBlendMode("ADD")
--> TOP
local topLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
topLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
topLine:SetThickness(2)
topLine:SetStartPoint("TOP", weizPVP.CrosshairFrame, 0, 2000)
topLine:SetEndPoint("TOP", weizPVP.CrosshairFrame, 0, -8)
topLine:SetVertexColor(1, 1, 1)
topLine:SetAlpha(lineAlpha)
topLine:SetBlendMode("ADD")
--> BOTTOM
local bottomLine = weizPVP.CrosshairFrame:CreateLine("BACKGROUND")
bottomLine:SetTexture("Interface\\addons\\weizPVP\\Textures\\white16x16.tga")
bottomLine:SetThickness(2)
bottomLine:SetStartPoint("BOTTOM", weizPVP.CrosshairFrame, 0, -2000)
bottomLine:SetEndPoint("BOTTOM", weizPVP.CrosshairFrame, 0, 8)
bottomLine:SetVertexColor(1, 1, 1)
bottomLine:SetAlpha(lineAlpha)
bottomLine:SetBlendMode("ADD")

--=> ARROWS
-------------------------------------------------------
local arrows = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
arrows:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/arrows.tga]])
arrows:SetVertexColor(1, 0, 0)
arrows:SetAlpha(arrowAlpha)
arrows:SetAllPoints()

--=> Create Net-O-Matic icon
-----------------------------------------------------------
local iconNetOMatic = weizPVP.CrosshairFrame:CreateTexture(nil, "BACKGROUND")
iconNetOMatic:SetTexture([[interface/addons/weizPVP/Plugins/Crosshair/netomatic.tga]])
iconNetOMatic:SetSize(24, 24)
iconNetOMatic:SetAlpha(1)
iconNetOMatic:SetPoint("BOTTOMLEFT", rightLine, "TOPLEFT", 0, 12)
iconNetOMatic:SetAlpha(0)

-------------------------------------------------------------------------------
--|> CREATE ANIMATIONS
-------------------------------------------------------------------------------
--=> ARROWS: Rotation Group
local ag = arrows:CreateAnimationGroup()
local rotation = ag:CreateAnimation("Rotation")
rotation:SetDegrees(-360)
rotation:SetDuration(5)
ag:SetLooping("REPEAT")
ag:Play()
--=> ARROWS: Fade In Group
local group = arrows:CreateAnimationGroup()
group:SetToFinalAlpha(true)

local AAG1 = group:CreateAnimation("Alpha")
AAG1:SetFromAlpha(0)
AAG1:SetToAlpha(1)
AAG1:SetDuration(0.2)

--=> ARROWS: Scale 2x
local scale1 = group:CreateAnimation("Scale")
scale1:SetScale(2, 2)
scale1:SetDuration(0.5)

--=> ARROWS: Scale to 50%
local scale50 = group:CreateAnimation("Scale")
scale50:SetScale(0.5, 0.5)
scale50:SetDuration(0.5)

--=> ARROWS: Fade Out
local fadeOut = weizPVP.CrosshairFrame:CreateAnimationGroup()
fadeOut:SetToFinalAlpha(true)
local AAG2 = fadeOut:CreateAnimation("Alpha")
AAG2:SetFromAlpha(1)
AAG2:SetToAlpha(0)
AAG2:SetDuration(0.2)
fadeOut:SetScript(
    "OnFinished",
    function(self)
        weizPVP.CrosshairFrame:Hide()
    end
)

--=> ARROWS: Fade in 2
local fadeIn = weizPVP.CrosshairFrame:CreateAnimationGroup()
fadeIn:SetToFinalAlpha(true)

local AAG3 = fadeIn:CreateAnimation("Alpha")
AAG3:SetOrder(1)
AAG3:SetFromAlpha(0)
AAG3:SetToAlpha(1)
AAG3:SetDuration(0.2)

fadeOut:SetScript(
    "OnFinished",
    function(self)
        weizPVP.CrosshairFrame:Hide()
    end
)

-------------------------------------------------------------------------------
--|> FUNCTIONS
-------------------------------------------------------------------------------
--=> ENABLE CROSSHAIR
-------------------------------------------------------
local function EnableCrosshair()
    --> Register Events
    weizPVP.CrosshairFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
    weizPVP.CrosshairFrame:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
    weizPVP.CrosshairFrame:RegisterEvent("NAME_PLATE_UNIT_ADDED")
    weizPVP.CrosshairFrame:RegisterEvent("DISPLAY_SIZE_CHANGED")
    weizPVP.CrosshairFrame:RegisterEvent("PLAYER_LOGIN")
    weizPVP.CrosshairFrame:RegisterEvent("PLAYER_ENTERING_WORLD")

    --> Activate if on enemy manually since there will be no events
    local nameplate = C_NamePlate.GetNamePlateForUnit("target")
    if nameplate and (UnitIsEnemy("player", "target") or DEBUG) and UnitIsPlayer("target") then
        targetPlate = nameplate
        weizPVP:FocusPlate(nameplate)
    else
        fadeOut:Play()
        targetPlate = nil
    end
end

--=> DISABLE CROSSHAIR
-------------------------------------------------------
local function DisableCrosshair()
    --> Clear plate and fade out
    targetPlate = nil
    fadeOut:Play()
    --> Unregister Events
    weizPVP.CrosshairFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
    weizPVP.CrosshairFrame:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
    weizPVP.CrosshairFrame:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
    weizPVP.CrosshairFrame:UnregisterEvent("DISPLAY_SIZE_CHANGED")
    weizPVP.CrosshairFrame:UnregisterEvent("PLAYER_LOGIN")
    weizPVP.CrosshairFrame:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

--=> Toggle Crosshair
-------------------------------------------------------
EnableCrosshair() --> Enabled by default
function weizPVP:ToggleCrosshair()
    if weizPVP.Options["Crosshair"].Enabled == true then
        weizPVP.Options["Crosshair"].Enabled = false
        print "|cffeaeaeaweiz|r|cffffa012PVP|r : |cff42dcf4Crosshair|r is now |cffff3838DISABLED|r"
        DisableCrosshair()
    else
        weizPVP.Options["Crosshair"].Enabled = true
        print "|cffeaeaeaweiz|r|cffffa012PVP|r : |cff42dcf4Crosshair|r is now |cff37ff37ENABLED|r"
        EnableCrosshair()
    end
end

--=> Set Color of lines and circle
-------------------------------------------------------
local function SetColor(r, g, b)
    circle:SetVertexColor(r, g, b)
    topLine:SetVertexColor(r, g, b)
    bottomLine:SetVertexColor(r, g, b)
    leftLine:SetVertexColor(r, g, b)
    rightLine:SetVertexColor(r, g, b)
end

--=> SCALE COORDS
-------------------------------------------------------
local xFactor, yFactor = 1, 1 -- pixel perfect stuff, just try and prevent it from screwing up our lines
local function ScaleCoords(xPixel, yPixel, trueScale)
    local x, y = xPixel / xFactor, yPixel / yFactor
    x, y = x - x % 1, y - y % 1 -- floor
    return trueScale and (xPixel * xFactor) or (x * xFactor), trueScale and (xPixel * xFactor) or (y * yFactor)
end

--=> Focus the "target" nameplate
-------------------------------------------------------
local Moving = false
function weizPVP:FocusPlate(plate)
    fadeOut:Stop()
    if not weizPVP.CrosshairFrame:IsShown() then
        local x, y = plate:GetCenter()
        if x and y then
            local scale = plate:GetEffectiveScale()
            x, y = x * scale, y * scale
            local fScale = weizPVP.CrosshairFrame:GetScale()
            x, y = x / fScale, y / fScale
            weizPVP.CrosshairFrame:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x, y))
        end
        fadeIn:Play()
    end

    weizPVP.CrosshairFrame:Show()
    group:Play()
    targetPlate = plate

    --> Set Class Colors
    local r, g, b = UnitSelectionColor("target")
    if UnitIsPlayer("target") then
        local class = UnitClass("target")
        class = string.upper(class)
        class = string.gsub(class, "%s", "")
        local classColor = RAID_CLASS_COLORS[class]
        if classColor then
            r, g, b = classColor.r, classColor.g, classColor.b
        else
            r, g, b = UnitSelectionColor("target")
        end
    end
    --> APPLY COLORS
    SetColor(r, g, b)
    Moving = GetTime()
end

--=> TARGET CHANGED
-------------------------------------------------------
function weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
    local nameplate = C_NamePlate.GetNamePlateForUnit("target") --weizPVP.CrosshairFrame:GetPlateByGUID(targetGUID)
    --> Activate on enemy
    if nameplate and (UnitIsEnemy("player", "target") or DEBUG) and UnitIsPlayer("target") then
        --> Fade out if not an enemy or no target
        targetPlate = nameplate
        weizPVP:FocusPlate(nameplate)
    else
        fadeOut:Play()
        targetPlate = nil
    end
end

--=> PLAYER ENTERING WORLD
-------------------------------------------------------
function weizPVP.CrosshairFrame:PLAYER_ENTERING_WORLD()
    self:PLAYER_TARGET_CHANGED()
end

--=> ON UPDATE
-------------------------------------------------------
weizPVP.CrosshairFrame:SetScript(
    "OnUpdate",
    function(self)
        topLine:SetStartPoint("TOP", weizPVP.CrosshairFrame, 0, UIParent:GetTop() - weizPVP.CrosshairFrame:GetTop())
        if targetPlate then
            --> Check if Net-o-Matic can be used on target
            if IsFlying("target") then
                local inRange = IsItemInRange("Net-o-Matic 5000", "target")
                local usable = IsUsableItem("Net-o-Matic 5000")
                if inRange and usable then
                    iconNetOMatic:SetAlpha(1)
                else
                    iconNetOMatic:SetAlpha(0)
                end
            end

            if Moving then
                local frame1, frame2 = weizPVP.CrosshairFrame, targetPlate
                local x1, y1 = frame1:GetCenter()
                x1, y1 = x1 * frame1:GetEffectiveScale(), y1 * frame1:GetEffectiveScale()
                local x2, y2 = frame2:GetCenter()
                x2, y2 = x2 * frame2:GetEffectiveScale(), y2 * frame2:GetEffectiveScale()
                local delta1, delta2 = y2 - y1, x2 - x1
                local distance = (delta2 ^ 2 + delta1 ^ 2) ^ 0.5

                local timeLeft = Moving + 1 - GetTime()

                if timeLeft > 0 and distance > 3 then
                    --> Move distance / timeLeft toward frame2
                    --> elapsed / timeLeft?
                    local amountToMove = distance / timeLeft / 10
                    local ratio = amountToMove / distance
                    --> Move a point along a line in a given direction
                    local x = x1 + ratio * delta2
                    local y = y1 + ratio * delta1
                    frame1:ClearAllPoints()
                    frame1:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x, y))
                else
                    frame1:ClearAllPoints()
                    frame1:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x2, y2))
                    Moving = false
                end
            else
                local x, y = targetPlate:GetCenter()
                local scale = targetPlate:GetEffectiveScale()
                x, y = x * scale, y * scale
                local fScale = weizPVP.CrosshairFrame:GetScale()
                x, y = x / fScale, y / fScale
                weizPVP.CrosshairFrame:SetPoint("CENTER", nil, "BOTTOMLEFT", ScaleCoords(x, y))
            end
        end
    end
)

--=> DISPLAY SIZE CHANGED
-------------------------------------------------------
function weizPVP.CrosshairFrame:DISPLAY_SIZE_CHANGED()
    local width, height = GetPhysicalScreenSize()
    xFactor, yFactor = (768 / width) * (width / height), 768 / height
end
function weizPVP.CrosshairFrame:PLAYER_LOGIN()
    weizPVP.CrosshairFrame:DISPLAY_SIZE_CHANGED()
end

--=> NAMEPLATE ADDED
-------------------------------------------------------
function weizPVP.CrosshairFrame:NAME_PLATE_UNIT_ADDED(unit)
    local nameplate = C_NamePlate.GetNamePlateForUnit(unit)
    if
        nameplate and UnitIsUnit("target", unit) and (UnitIsEnemy("player", "target") or DEBUG) and
            UnitIsPlayer("target")
     then
        targetPlate = nameplate
        weizPVP:FocusPlate(nameplate)
    end
end

--=> NAMEPLATE REMOVED
-------------------------------------------------------
function weizPVP.CrosshairFrame:NAME_PLATE_UNIT_REMOVED(unit)
    if UnitIsUnit("target", unit) then
        targetPlate = nil
        fadeOut:Play()
    end
end

--=> CrosshairFrame OnEvent
-------------------------------------------------------
weizPVP.CrosshairFrame:SetScript(
    "OnEvent",
    function(self, event, ...)
        return self[event] and self[event](self, ...)
    end
)
